'use strict';

var extend = require( 'extend' );
var http = require( 'http' );
var url = require( 'url' );

module.exports = function( _options, callback ) {
    var options = extend( true, {
        url: '//' + ( _options.host ? _options.host : ( _options.hostname + ( _options.port ? ':' + _options.port : '' ) ) ) + _options.path,
        responseType: 'application/json',
        headers: {},
        withCredentials: false
    }, _options );
    
    var data = options.data;
    delete options.data;
    
    var parsedURI = url.parse( options.url, false, true );
    delete options.url;
    
    options.hostname = options.hostname || parsedURI.hostname;
    options.port = options.port || parsedURI.port;
    options.path = options.path || parsedURI.path;
    
    var dataString;
    if ( typeof( data ) !== 'undefined' ) {
        if ( options.method.toLowerCase() === 'get' ) {
            var pairs = [];
            for ( var key in data ) {
                pairs.push( encodeURIComponent( key ) + '=' + encodeURIComponent( data[ key ] ) );
            }
            
            options.path += ( ( options.path.indexOf( '?' ) !== -1 ) ? '&' : '?' ) + pairs.join( '&' );
        }
        else {
            try {
                dataString = JSON.stringify( data );
                options.headers[ 'Content-Type' ] = 'application/json';
                options.headers[ 'Content-Length' ] = dataString.length;
            }
            catch( ex ) {
                if ( callback ) {
                    callback( ex );
                }
                return;
            }        
        }
    }
    
    var buffer;
    var request = http.request( options, function( response ) {

        response.on( 'error', function( error ) {
            if ( callback ) {
                callback( error );
            }
        } );
        
        response.on( 'data', function( _buffer ) {
            buffer = buffer || '';
            buffer += _buffer;
        } );
        
        response.on( 'end', function() {
            try {
                var obj = JSON.parse( buffer );
                if ( callback ) {
                    callback( null, obj );
                }
            }
            catch( ex ) {
                if ( callback ) {
                    callback( ex, buffer );
                }
            }
        } );
    } );
    
    if ( dataString ) {
        request.write( dataString );
    }
    
    request.end();
};